/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_BUFFER_H_INCLUDED)
#define HARK_BASE_BUFFER_H_INCLUDED 1

#include "common.h"

#include "Object.h"
#include "ObjectRef.h"
#include "Exception.h"
#include <typeinfo>
#include <vector>

namespace HARK
{
    class Buffer : public Object
    {
    public:
	Buffer(int bLength);

	Buffer(const Buffer&);

	ObjectRef &get(int ind) const;
	ObjectRef &operator[] (int ind);
	int isValid(int ind) const;
	void printOn(std::ostream &out = std::cout) const;

	int getCurrentPos();

    protected:
	mutable std::vector<ObjectRef> data;
	mutable std::vector<int>       flags;
	int         bufferLength;
	mutable int bufferPos;
	mutable int currentPos;

    };

    class BufferException : public BaseException
    {
    public:
	BufferException(const Buffer *_buffer, std::string _message, int _element);
	virtual void print(std::ostream &out = std::cerr) const;

    protected:
	const Buffer *buffer;
	std::string message;
	int element;
    };
}

#endif

/* end of file */
