/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_MAP_H_INCLUDED)
#define HARK_BASE_MAP_H_INCLUDED 1

#include "Object.h"
#include "ObjectRef.h"
#include <map>

namespace HARK
{
    template <class _Key, class _Tp>
    class Map : public Object , public std::map<_Key, _Tp> {
    public:
	Map();
	Map(const Map &m);

	virtual void printOn(std::ostream& out = std::cout) const;
	virtual ObjectRef clone();
    };

    template<typename K, typename T>
    Map<K, T>::Map()
    {
    }

    template<typename K, typename T>
    Map<K, T>::Map(const Map<K, T> &m) :
	std::map<K, T>(m)
    {
    }

    template<typename K, typename T>
    void
    Map<K, T>::printOn(std::ostream &out) const
    {
    }

    template<typename K, typename T>
    ObjectRef
    Map<K, T>::clone()
    {
	ObjectRef cpy = ObjectRef(new Map<K, T>);
	Map<K, T> &mpOr = object_cast<Map<K, T> > (cpy);
	for (typename Map<K, T>::const_iterator it = this->begin(); it != this->end(); ++it) {
	    mpOr.insert(std::make_pair(it->first, it->second));
	}
	return cpy;
    }
    
    template <>
    inline void
    Map<int, ObjectRef>::printOn(std::ostream &out) const
    {
	if (this->empty() == true) {
	    out << "Map empty" << std::endl;
	}
	else {
	    Map<int, ObjectRef>::const_iterator it = this->begin();
	    out << "Map begins from " << (it->first) << std::endl;
	    out << "Number of channels is " << this->size() << std::endl;
	    for (; it != this->end(); ++it) {
		out << "Map's ObjectRef's channel " << (it->first) << " is:\n" << (it->second) << std::endl;
	    }
	}
    }

    template<>
    inline ObjectRef
    Map<int, ObjectRef>::clone()
    {
	ObjectRef cpy = ObjectRef(new Map<int, ObjectRef>);
	Map<int, ObjectRef> &mpOr = object_cast<Map<int, ObjectRef> > (cpy);
	for (Map<int, ObjectRef>::const_iterator it = this->begin(); it != this->end(); ++it) {
	    mpOr.insert(std::make_pair(it->first, it->second));
	}
	return cpy;
    }
}

#endif

/* end of file */

