/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_NODEFACTORY_H_INCLUDED)
#define HARK_BASE_NODEFACTORY_H_INCLUDED 1

#include "common.h"

#include "ParameterSet.h"
#include <vector>
#include <string>

namespace HARK
{
    class Node;

    class _NodeFactory
    {
    protected:
	std::string name;

    public:
	_NodeFactory(std::string _name) :
	    name(_name)
	{}

	virtual const std::string &getName()
	{
	    return name;
	}

	virtual Node* Create(const std::string &name, const ParameterSet &parameters) = 0;
	virtual ~_NodeFactory(){}
	
    };

    template <class T>
    class NodeFactory : public _NodeFactory
    {
    public:
	NodeFactory(std::string _name) :
	    _NodeFactory(_name)
	{}

	virtual Node* Create(const std::string &name, const ParameterSet &parameters)
	{
	    return ((Node*) new T(name,parameters));
	}
    };
}

#endif

/* end of file */
