/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_OBJECTREF_H_INCLUDED)
#define HARK_BASE_OBJECTREF_H_INCLUDED 1

#include "common.h"

#include "Object.h"
#include "net_types.h"
#include "conversion.h"
#include "BaseException.h"
#include <typeinfo>
#include "pointer.h"

namespace HARK
{
    extern ObjectRef nilObject;

    template <class X>
    template <class Z>
    RCPtr<X>& RCPtr<X>::operator= (const RCPtr<Z> &r)
    {
	if ((void*) this != (void*) (&r)) {
	    X *tmp=dynamic_cast<X*> (r.ptr);
	    if (!tmp) {

		RCPtr<Object> conv = Conversion::convertTo<X>(r);

		tmp = dynamic_cast<X*>(conv.ptr);

		if (!tmp) {
		    throw new GeneralException("Something is wrong in RCPtr::operator=, this should not happen.",__FILE__,__LINE__);
		}

		release();
		ptr=tmp;
		acquire();
	    } else {
		release();
		ptr=tmp;
		acquire();
	    }
	}
	return *this;
    }

    template <class X>
    template <class Z>
    RCPtr<X>::RCPtr (const RCPtr<Z> &r)
    {
	X *tmp=dynamic_cast<X*> (r.ptr);
	if (!tmp) {

	    RCPtr<Object> conv = Conversion::convertTo<X>(r);
	    tmp = dynamic_cast<X*>(conv.ptr);

	    if (!tmp) {
		throw new GeneralException("Something is wrong in RCPtr::operator=, this should not happen.",__FILE__,__LINE__);
	    }

	    ptr=tmp;
	    acquire();
	} else {
	    ptr=tmp;
	    acquire();
	}
    }

    template <class T>
    class CastException : public GenericCastException
    {
    public:
	CastException(std::string _type) : type(_type)
	{}

	virtual void print(std::ostream &out = std::cerr) const
	{
	    out << "Cast error: Trying to cast ObjectRef (" << type << ") into " << typeid(T).name() << std::endl;
	}

    protected:
	std::string type;
    };

    template <class T>
    inline T object_ptr_cast (const ObjectRef &ref)
    {
	T tmp = dynamic_cast<T>(&(*ref));
	if (!tmp) {
	    throw new CastException<T> (typeid ((*ref)).name());
	}
	return tmp;
    }

    template <class T>
    inline T &object_cast (const ObjectRef &ref)
    {
	T *tmp = dynamic_cast<T *>(&(*ref));
	if (!tmp) {
	    throw new CastException<T> (typeid ((*ref)).name());
	}
	return *tmp;
    }

    template <class T>
    inline T &dereference_cast (const ObjectRef &ref)
    {
	GenericType<T> *tmp = (dynamic_cast<GenericType<T> * >(&(*ref)));
	if (!tmp) {
	    throw new CastException<T> (typeid ((*ref)).name());
	}
	return tmp->val();
    }

    template <class T>
    inline T *object_kind_of (const ObjectRef &ref)
    {
	return dynamic_cast<T *>(&(*ref));
    }

    template <class T>
    inline bool object_has_type (const ObjectRef &ref)
    {
	return typeid(*ref) == typeid(T);
    }
}

#endif

/* end of file */
