/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_PARAMETERSET_H_INCLUDED)
#define HARK_BASE_PARAMETERSET_H_INCLUDED 1

#include "common.h"

#include "Object.h"
#include <map>
#include <string>
#include "BaseException.h"

namespace HARK
{
    class ParameterSet : public std::map<std::string,std::pair<ObjectRef,bool> >
    {
    public:
	bool exist(const std::string &param) const;
	ObjectRef get(const std::string &param) const;
	ObjectRef getDefault(const std::string &param, ObjectRef value);
	void defaultParam(const std::string &param, ObjectRef value);
	void add(const std::string &param, ObjectRef value);
	void print(std::ostream &out = std::cerr) const;
	void checkUnused() const;
    };

    class ParameterException : public BaseException
    {
    public:
	ParameterException(std::string _message, std::string _param_name, ParameterSet _params) :
	    param_name(_param_name),
	    params(_params),
	    message(_message)
	{}

	virtual void print(std::ostream &out = std::cerr) const
	{
	    out << message << ": "<< param_name <<std::endl;
	    out << "Given parameters are:\n";
	    params.print(out);
	}
    protected:
	std::string param_name;
	ParameterSet params;
	std::string message;
    };

}

#endif

/* end of file */
