/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_SOURCE_H_INCLUDED)
#define HARK_BASE_SOURCE_H_INCLUDED 1

#include "Object.h"

namespace HARK
{
    class Source : public Object {
    public:
	enum TFDim {
	    TFHeight,
	    TFDirection,
	    TFRange
	};
	enum CompareMode {
	    DEG,
	    TFINDEX
	};

    public:
	Source();
	Source(const Source &src);

    public:
	int tfindexDistance(const int *y) const;
	int tfindexDistance(const Source &src) const;

	float angle(const float *y) const;
	float angle(const Source &src) const;

	virtual ObjectRef clone();
	virtual void printOn(std::ostream &out) const;
	void readFrom(std::istream &in);

    public:
	float x[3];
	float power;
	int id;
	float remaining;
	int count;
	int last_found_count;

	int tfindex[3];
	int tfindex_max[3];
	int tfindex_min[3];
	CompareMode compare_mode;
    };

    std::istream &operator >> (std::istream &in, Source &src);
}

#endif

/* end of file */
