/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_TIME_H_INCLUDED)
#define HARK_BASE_TIME_H_INCLUDED 1

#include <chrono>

#include "common.h"

#include "Object.h"
#include "ObjectParser.h"
#include "misc.h"
#include "_strTimelog.hpp"

namespace HARK
{
    class Time : public Object {
    public:
        std::chrono::system_clock::time_point time;

    public:
        Time() {}
        Time(std::chrono::system_clock::time_point t) : time(t) {}

        void setTime(const std::chrono::system_clock::time_point& t) {
            time = t;
        }

        void setLocalTime() {
            time = std::chrono::system_clock::now();
        }

        virtual void printOn(std::ostream& out) const
        {
            out << "<Time <time " << _TimeLog(time) << "> >";
        }

        void readFrom(std::istream &in)
        {
        }

        friend std::istream& operator>>(std::istream &in, Time& src);
    };

    inline std::istream& operator>>(std::istream &in, Time& src)
    {
        if (!isValidType(in, "Time")) {
            return in;
        }
        src.readFrom(in);
        return in;
    }
}

#endif
