/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_TIMESTAMP_H_INCLUDED)
#define HARK_BASE_TIMESTAMP_H_INCLUDED 1

#include "common.h"

#include <sys/time.h>

#include "Object.h"
#include "ObjectParser.h"
#include "misc.h"

namespace HARK
{
    class TimeStamp : public Object {
    public:
        struct timeval time;

    public:
        TimeStamp() {
            time.tv_sec = 0;
            time.tv_usec = 0;
        }
        TimeStamp(struct timeval t) : time(t) {}
        TimeStamp(long int sec, long int usec) {
            time.tv_sec = sec;
            time.tv_usec = usec;
        }

        void setTime(struct timeval t) {
            time = t; 
        }
        void setTime(long int sec, long int usec) {
            time.tv_sec = sec;
            time.tv_usec = usec;
        }

        void setLocalTime() {
            gettimeofday(&time, NULL);
        }

        virtual void printOn(std::ostream& out) const
        {
            out << "<TimeStamp <sec " << time.tv_sec << "> <usec " << time.tv_usec << "> >";
        }

        void readFrom(std::istream &in)
        {
        }
  
        friend std::istream& operator>>(std::istream &in, TimeStamp& src);
    };

    inline std::istream& operator>>(std::istream &in, TimeStamp& src)
    {
        if (!isValidType(in, "TimeStamp")) {
            return in;
        }
        src.readFrom(in);
        return in;
    }
}

#endif
