/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "BaseException.h"

namespace HARK
{
    BaseException::~BaseException()
    {
    }
    
    void
    BaseException::freeze()
    {
    }

    BaseException *
    BaseException::add(BaseException *e)
    {
	return (new ExceptionStack)->add(this)->add(e);
    }

    GeneralException::GeneralException(std::string _message, std::string _file, int _line) :
	message(_message),
	file(_file),
	line(_line)
    {
    }

    void
    GeneralException::print(std::ostream &out) const
    {
	out << file << " line " << line << ": " << message << std::endl;
    }

    ExceptionStack::ExceptionStack()
    {
    }

    ExceptionStack::~ExceptionStack()
    {
	for (size_t i=0; i<stack.size(); i++) {
	    delete stack[i];
	}
    }

    BaseException *
    ExceptionStack::add(BaseException *e)
    {
	stack.insert(stack.end(), e);
	return this;
    }

    void
    ExceptionStack::print(std::ostream &out) const
    {
	for (unsigned int i=0; i<stack.size(); i++) {
	    stack[i]->print(out);
	}
    }

    void
    ExceptionStack::freeze()
    {
	for (unsigned int i=0; i<stack.size(); i++) {
	    stack[i]->freeze();
	}
    }
}

/* end of file */
