/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "Constant.h"

namespace HARK
{
    DECLARE_NODE(Constant);
    /*Node
     *
     * @name Constant
     * @category General
     * @description Defines a constant in terms of type and value. The different types are: int, float, bool, string, objects and subnet_param.
     *
     * @output_name VALUE
     * @output_description The value (parameter)
     *
     * @parameter_name VALUE
     * @parameter_description The value
     *
     END*/

    Constant::Constant(const std::string &nodeName, const ParameterSet &params) :
	Node(nodeName, params)
    {
	this->_outputID = addOutput("VALUE");
    }

    void
    Constant::initialize()
    {
	this->_value = parameters.get("VALUE");
	this->Node::initialize();
    }

    ObjectRef
    Constant::getOutput(int output_id, int count)
    {
	if (output_id == this->_outputID) {
	    return this->_value;
	} else {
	    throw new NodeException (this, "Constant: Unknown output id", __FILE__, __LINE__);
	}
    }
}

/* end of file */
