/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "Equal.h"

#include "operators.h"

namespace HARK
{
    DECLARE_NODE(Equal);
    /*Node
     *
     * @name Equal
     * @category Operator
     * @description Returns true if two input values are equal, false otherwise
     *
     * @input_name INPUT1
     * @input_description First value
     *
     * @input_name INPUT2
     * @input_description Second value
     *
     * @output_name OUTPUT
     * @output_type bool
     * @output_description True or false
     *
     END*/

    Equal::Equal(const std::string &nodeName, const ParameterSet &params) :
        BufferedNode(nodeName, params)
    {
        this->_input1ID = this->addInput("INPUT1");
        this->_input2ID = this->addInput("INPUT2");
        this->_outputID = this->addOutput("OUTPUT");
    }

    void
    Equal::calculate(int output_id, int count, Buffer &out)
    {
        ObjectRef input1Value = this->getInput(this->_input1ID, count);
        ObjectRef input2Value = this->getInput(this->_input2ID, count);
        out[count] = (input1Value == input2Value);
    }
}

/* end of file */
