/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "Iterate.h"

namespace HARK
{
    DECLARE_NODE(Iterate);
    /*Node

     * @name Iterate
     * @category Flow
     * @description Specify the number of iteration to do (max). Therefore, it can only be used in iterators and the output must be set to COND (left click on the node output and hold the control).

     * @output_name OUTPUT
     * @output_description Return true if count < MAX_ITER
     * @output_type bool

     * @parameter_name MAX_ITER
     * @parameter_description Number of iteration to do (max)
     * @parameter_type int

     END*/

    Iterate::Iterate(const std::string &nodeName, const ParameterSet &params) :
	Node(nodeName, params)
    {
        this->_outputID    = this->addOutput ("OUTPUT");
        this->_trueObject  = ObjectRef(new Bool(true));
        this->_falseObject = ObjectRef(new Bool(false));
        this->_maxIter     = dereference_cast<int> (parameters.get("MAX_ITER"));
    }

    ObjectRef
    Iterate::getOutput(int output_id, int count)
    {
        if (count < this->_maxIter) {
            return this->_trueObject;
        } else {
            return this->_falseObject;
        }
    }

}

/* end of file */
