/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "Object.h"

#include "ObjectParser.h"
#include "Exception.h"

namespace HARK
{
    std::map<std::string, ModuleInfo> Object::module_info_dic;

    std::map<std::string, _ObjectFactory *> &
    Object::ObjectFactoryDictionary()
    {
	static std::map<std::string, _ObjectFactory *> dict;
	return dict;
    }

    TypeMap<_ObjectFactory *>&
    Object::TypeidDictionary()
    {
	static TypeMap<_ObjectFactory *> dict;
	return dict;
    }

    std::string
    Object::className() const
    {
	TypeMap<_ObjectFactory *> &m = TypeidDictionary();
	TypeMap<_ObjectFactory *>::iterator found = m.find(&typeid(*this));
	if (found != m.end()) {
	    return found->second->getName();
	} else {
	    return "Object::className(): unknown";
	}
    }

    ObjectRef
    Object::newObject(const std::string &objType)
    {
	if (ObjectFactoryDictionary().find(objType) != ObjectFactoryDictionary().end()) {
	    return ObjectFactoryDictionary()[objType]->create();
	} else {
	    throw new GeneralException ("ObjectRef error: unknown type " + objType, __FILE__, __LINE__);
	}
    }

    void
    Object::serialize(std::ostream &out) const
    {
	printOn(out);
    }

    void
    Object::unserialize(std::istream &in)
    {
	throw new GeneralException("Object doesn't know how to unserialize itself", __FILE__, __LINE__);
    }

    void
    Object::doesNotUnderstand(const std::string &method)
    {
	throw new GeneralException(std::string(typeid (*this).name()) + ": Does Not Understand method " + method, __FILE__, __LINE__);
    }

    int
    Object::setModuleVersion(const std::string &module_name, const std::string &svn, const std::string &module)
    {
	if (module_info_dic.count(module_name)) {
	    std::cerr << "# Error setMoudleVersion(): " << module_name << " already exists.";
	    return -1;
	}
	module_info_dic[module_name].svn_version = svn;
	module_info_dic[module_name].module_version = module;

	return 0;
    }

    ObjectRef nilObject = ObjectRef(new NilObject);
}

/* end of file */
