/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "multithread.h"

namespace HARK
{
    AtomicCounter::AtomicCounter(int c) :
	count(c)
    {
	pthread_mutex_init(&this->mutex, NULL);
    }

    AtomicCounter::~AtomicCounter()
    {
	pthread_mutex_destroy(&this->mutex);
    }

    void
    AtomicCounter::inc()
    {
	pthread_mutex_lock(&this->mutex);
	++this->count;
	pthread_mutex_unlock(&this->mutex);
    }

    bool
    AtomicCounter::dec()
    {
	int tmp;
	pthread_mutex_lock(&this->mutex);
	--this->count;
	tmp = this->count;
	pthread_mutex_unlock(&this->mutex);
	return (tmp > 0);
    }

    bool
    AtomicCounter::unique() const
    {
	return this->count == 1;
    }

}

/* end of file */
