/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "object_param.h"

#include "ObjectParser.h"
#include <string>
#include <sstream>

namespace HARK
{
    const std::vector<std::string> &
    ObjectParam::allTypes(bool allowSubnetParam)
    {
	static std::vector<std::string> types;
	static std::vector<std::string> Stypes;

	static bool init = false;
	if (!init) {
	    types.push_back("int");
	    types.push_back("float");
	    types.push_back("string");
	    types.push_back("bool");
	    types.push_back("object");

	    Stypes.push_back("int");
	    Stypes.push_back("float");
	    Stypes.push_back("string");
	    Stypes.push_back("bool");
	    Stypes.push_back("object");
	    Stypes.push_back("subnet_param");
	    init = true;
	}

	if (allowSubnetParam) {
	    return Stypes;

	} else {
	    return types;
	    
	}
    }

    ObjectRef
    ObjectParam::stringParam(const std::string &type, const std::string &value, const ParameterSet &param)
    {
	if (value == "") {
	    return ObjectRef();
	}

	if (type == "int") {
	    int val = 0;
	    std::stringstream tempStream(value);
	    tempStream >> val;
	    return ObjectRef(Int::alloc(val));

	} else if (type == "bool") {
	    if (value == "true" || value == "TRUE") {
		return ObjectRef (new Bool(true));

	    } else if (value == "false" || value == "FALSE") {
		return ObjectRef(new Bool(false));

	    } else {
		std::cerr << "value: " << value << std::endl;
		throw new GeneralException("Bool value isn't either true or false... make up your mind",
					   __FILE__, __LINE__);
	    }

	} else if (type == "float") {
	    float val = 0;

	    std::stringstream tempStream(value);
	    tempStream >> val;

	    return ObjectRef(Float::alloc(val));

	} else if (type == "string") {
	    return ObjectRef(new String(value));

	} else if (type == "object") {
	    std::istringstream obj(value);
	    ObjectRef val;
	    obj >> val;
	    return val;

	} else if (type == "expr") {
	    throw new GeneralException("Expressions not supported yet in params", __FILE__, __LINE__);

	} else if (type == "subnet_param") {
	    if (param.exist(value)) {
		return param.get(value);

	    } else {
		return ObjectRef();

	    }

	} else {
	    throw new GeneralException("Unknown parameter type: \"" + type + "\"", __FILE__, __LINE__);

	}
    }

}

/* end of file */
